package mv3302.run;

import static java.lang.Math.sqrt;
import mv3302.PartArrivalProcess;
import mv3302.TransferLineComponent;
import simkit.Schedule;
import simkit.random.RandomVariate;
import simkit.random.RandomVariateFactory;
import simkit.stat.SimpleStatsTally;
import simkit.stat.StudentT;
import simkit.stat.TruncatingSimpleStatsTally;

/**
 * The main class for running steady state analysis.
 *
 * @author dansl
 */
public class RunSteadyStateAnalysis {

    /**
     * The main method for executing the steady state analysis.
     *
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        // Create a random variate generator for interarrival times with an exponential distribution and mean of 2.5
        RandomVariate interarrivalTimeGenerator = RandomVariateFactory.getInstance(
                "Exponential", 2.5);
        // Create a part arrival process with the interarrival time generator
        PartArrivalProcess arrivalProcess = new PartArrivalProcess(interarrivalTimeGenerator);
        // Create an array of random variate generators for processing times
        RandomVariate[] processingTimeGenerator
                = {RandomVariateFactory.getInstance("Exponential", 2.4),
                    RandomVariateFactory.getInstance("Gamma", 3.2, 3.3),
                    RandomVariateFactory.getInstance("Uniform", 4.5, 6.7),
                    RandomVariateFactory.getInstance("Exponential", 3.0)};
        // Create an array of total number of workstations
        int[] totalNumberWorkstations = new int[4];
        totalNumberWorkstations[0] = 1;
        totalNumberWorkstations[1] = 5;
        totalNumberWorkstations[2] = 4;
        totalNumberWorkstations[3] = 2;
        // Create a transfer line component with the processing time generators and total number of workstations
        TransferLineComponent transferLineComponent = new TransferLineComponent(processingTimeGenerator, totalNumberWorkstations);
        // Add the transfer line component as a listener to the arrival process
        arrivalProcess.addSimEventListener(transferLineComponent);
        int warmup = 1000;
        int observations = 10000;
        // Create a statistical tally for tracking delay in queue
        SimpleStatsTally delayInQueueStat = new TruncatingSimpleStatsTally("delayInQueue", 2000);
        // Add the delay in queue stat as a property change listener to the transfer line component
        transferLineComponent.addPropertyChangeListener(delayInQueueStat);
        System.out.println(arrivalProcess);
        System.out.println(transferLineComponent);
        // Create a statistical tally for tracking delay in queue in the outer loop
        SimpleStatsTally outerDelayInQueueStat = new SimpleStatsTally("outerDelayInQueue");
        // Stop the simulation after warmup + observations arrivals
        Schedule.stopOnEvent(warmup + observations, "Arrival");
        int numberReplications = 50;
        double alpha = 0.05;
        // Outer loop for multiple replications
        System.out.println("Avg Delay In Queue");
        for (int replication = 1; replication <= numberReplications; ++replication) {
            // Reset the simulation schedule and delay in queue stat for each replication
            Schedule.reset();
            delayInQueueStat.reset();
            Schedule.startSimulation();
            // Record the mean delay in queue for the replication in the outer tally
            outerDelayInQueueStat.newObservation(delayInQueueStat.getMean());
            // Print the replication number, average delay, and confidence interval
            System.out.printf("%d %.3f ± %.3f%n",
                    replication,
                    outerDelayInQueueStat.getMean(),
                    outerDelayInQueueStat.getStandardDeviation()
                    * StudentT.getQuantile(1.0 - 0.5 * alpha, outerDelayInQueueStat.getCount() - 1) / sqrt(outerDelayInQueueStat.getCount())
            );
        }
    }
}
